package jcg.zheng.demo.junit5;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.util.Arrays;
import java.util.List;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import jcg.zheng.demo.SomeException;
import jcg.zheng.demo.SomeClass;
import jcg.zheng.demo.OtherService;
import jcg.zheng.demo.SomeInterface;

@ExtendWith(MockitoExtension.class)
public class SomeClassTest {

	@InjectMocks
	private SomeClass classUnderTest;

	@Mock
	private SomeInterface mockedInterface;

	@Mock
	private OtherService mockedService;

	@Test
	@DisplayName("It should double a number")
	public void test_doubleANumber() {
		when(mockedService.calculateByDoubleIt(3)).thenReturn(6);

		assertEquals(6, classUnderTest.doubleANumber(3));
		verify(mockedService, times(1)).calculateByDoubleIt(3);
	}

	@Test
	public void test_doubleANumber_exception() {
		when(mockedService.calculateByDoubleIt(3)).thenThrow(NullPointerException.class);

		assertThrows(SomeException.class, () -> {
			classUnderTest.doubleANumber(3);
		});

		verify(mockedService, times(1)).calculateByDoubleIt(3);
	}

	@Test
	public void test_processData() {
		when(mockedInterface.getSomeList(anyString())).thenReturn(null);
		classUnderTest.processData("NA");

		verify(mockedInterface, times(1)).getSomeList(anyString());
		verify(mockedService, times(0)).doSomething(anyString());
	}

	@Test
	public void test_processData_2() {
		List<String> twoItemsList = Arrays.asList("Mary", "Zheng");
		when(mockedInterface.getSomeList(anyString())).thenReturn(twoItemsList);
		classUnderTest.processData("NA");

		verify(mockedInterface, times(1)).getSomeList(anyString());
		verify(mockedService, times(2)).doSomething(anyString());
	}

	@Test
	public void test_returnBooleanFoo_exception() {
		when(mockedService.returnABoolean("NA")).thenThrow(NullPointerException.class);
		assertThrows(SomeException.class, () -> {
			classUnderTest.returnABoolean("NA");
		});

		verify(mockedService, times(1)).returnABoolean("NA");
	}

	@Test
	public void test_returnBooleanFoo_false() {
		when(mockedService.returnABoolean("NA")).thenReturn(false);

		boolean shouldReturnFalse = classUnderTest.returnABoolean("NA");
		assertFalse(shouldReturnFalse);
	}

	@Test
	public void test_returnBooleanFoo_true() {
		when(mockedService.returnABoolean("Save")).thenReturn(true);

		boolean shouldReturnTrue = classUnderTest.returnABoolean("Save");
		assertTrue(shouldReturnTrue);
		verify(mockedService, times(1)).returnABoolean("Save");
	}

	@Test
	public void test_voidFoo() throws IllegalAccessException {

		classUnderTest.voidFoo("OK");
		verify(mockedService, times(1)).doSomething("OK");

	}

	@Test
	public void test_voidFoo_exception() {
		doThrow(IllegalStateException.class).when(mockedService).doSomething("NA");
		assertThrows(SomeException.class, () -> {
			classUnderTest.voidFoo("NA");
		});

	}

	@Test
	public void test_voidFoo_interface() throws IllegalAccessException {

		classUnderTest.voidFoo("Interface");
		verify(mockedService, times(1)).doSomething("Interface");
		verify(mockedInterface, times(1)).doSomething("Interface");

	}

}
